#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import yaml

def check_file_format(file_path):
    """
    Check if the file is in valid yaml format
    """
    try:
        with open(file_path, 'r') as config_file:
            yaml.safe_load(config_file)
        return True
    except (yaml.parser.ParserError, yaml.scanner.ScannerError) as e:
        print(f"Error: {file_path} is not in valid yaml format. {e}")
        return False

def get_keys(config):
    """
    Returns the keys of a valid yaml file, or None if the file is not found or not in yaml format
    """
    return list(config.keys()) if config else None

def check_and_ask(config):
    """
    Recursively check for undefined keys and ask for input
    """
    if isinstance(config, dict):
        for key in config:
            if config[key] is None:
                config[key] = input(f"Please enter a value for {key}: ")
            elif isinstance(config[key], dict):
                check_and_ask(config[key])

def parse_config():
    config_files = [
        os.path.join(os.path.expanduser("~"), ".sd-upload-config.yml"),
        os.path.join(os.path.expanduser("~"), ".sd-upload-config.yaml"),
        "sd-upload-config.yml",
        "sd-upload-config.yaml"
    ]

    config = None
    file = None

    # Set variable config_file to the first config file found
    for cfg_file in config_files:
        if os.path.isfile(cfg_file) and check_file_format(cfg_file):
            with open(cfg_file, 'r') as config_file:
                config = yaml.safe_load(config_file)
                file = cfg_file
                break

    if not config:
        print("Error: No valid yaml config file found.")
        return None

    check_and_ask(config)

    # write the config file with the new values
    try:
        with open(file, "w") as config_file:
            yaml.dump(config, config_file, allow_unicode=True, width=float("inf"))
    except (OSError, yaml.parser.ParserError, yaml.scanner.ScannerError) as e:
        print(f"Error: Failed to write config file. {e}")

    return config



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import requests
import json

from robin_sd_upload.api_interaction import get_bearer_token
from robin_sd_upload.supportive_scripts import yaml_parser
from robin_sd_upload.supportive_scripts import logger

def push_software(fpath, radarType, version_name):
    config = yaml_parser.parse_config()

    request_url = config['api_url']

    bearer_token = str(get_bearer_token.get_bearer_token())

    headers = {
        'Authorization': 'Bearer ' + bearer_token,
    }

    # if not bearer_token:
    #     return "Bearer token is empty"
    # else:
    #     print('bearer_token: ', bearer_token)

    # check if can open file path
    if os.path.isfile(fpath):
        logger.log(message="ZIP file exists: " + fpath, log_level="info", to_file=True, to_terminal=True)
        # print("ZIP file exists: " + fpath)
    else:
        logger.log(message="ZIP not exist (PS): " + fpath, log_level="error", to_file=True, to_terminal=True)
        # print("ZIP not exist: " + fpath)
        return "ZIP not exist (PS): " + fpath
        
    files = {
        'file': (fpath, open(fpath, 'rb'))
    }

    values = {
        'destination': json.dumps(radarType),
        'versionName': json.dumps(version_name)
    }
    
    # print versionName
    # print('versionName: ', version_name)
    # print('values.versionName: ', values['versionName'])

    logger.log(message="Pushing software to Robin SD", log_level="info", to_file=True, to_terminal=True)

    response = requests.post(request_url + '/api/softwares/softwarefiles', headers=headers, data=values, files=files)
    logger.log(message="Response code: " + str(response.status_code), log_level="info", to_file=True, to_terminal=True)
    return response.json()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import tempfile
import argparse

from robin_sd_upload import _version
from robin_sd_upload.slack_interaction import slack_handler
from robin_sd_upload.supportive_scripts import yaml_parser
from robin_sd_upload.supportive_scripts import logger

from robin_sd_upload.api_interaction import push_software
from robin_sd_upload.supportive_scripts import check_upload_file
from robin_sd_upload.supportive_scripts import validate
from robin_sd_upload.supportive_scripts import create_zip
from robin_sd_upload.supportive_scripts import remove_zip

def arg_parser():
    parser = argparse.ArgumentParser(
        description='Robin Radar Systems Software Uploader',
        usage='robin-sd-upload [options]',
        prog='Robin Radar Systems Software Uploader',
        epilog='To report any bugs or issues, please visit: \
        https://support.robinradar.systems or run: robin-sd-upload --slack'
    )

    parser.add_argument('-c', '--check', action='store_true', help='ensure all prerequisites are met')
    parser.add_argument('-u', '--upload', action='store_true', help='upload software: robin-sd-upload --upload --type=radar_type --version=version_name --source=upload_file_folder_path')
    parser.add_argument('-v', '--version', action='version', version='%(prog)s {version}'.format(version=_version.__version__))
    parser.add_argument('-s', '--slack', action='store_true', help='Send the logs to IT/DevOps Slack channel')

    # arguments for upload
    parser.add_argument('-t', "--type", type=str, help="radar type")
    parser.add_argument("-n", "--number", type=str, help="version number of the software")
    parser.add_argument('-p', "--path", type=str, help="upload file absolute path")

    args = parser.parse_args()
    config = yaml_parser.parse_config()

    logger.log(message="Starting Robin Radar Systems Software Uploader", log_level="info", to_file=True, to_terminal=True)
    logger.log(message="Version: " + _version.__version__, log_level="info", to_file=True, to_terminal=True)
    logger.log(message="Username: " + config['robin_email'], log_level="info", to_file=True, to_terminal=True)

    radarType = args.type
    version_name = args.number
    upload_file_path = args.path

    # print('radarType: ', radarType)
    # print('version_name: ', version_name)
    # print('upload_file_path: ', upload_file_path)

    if args.check:
        # check if all the prerequisites are met
        logger.log(message="All prerequisites met.", log_level="info", to_file=True, to_terminal=True)
        exit(0)
    elif args.upload:
        if radarType is None or version_name is None or upload_file_path is None:
            print('Please provide all the arguments: --type, --version, --path')
            exit(1)
        # check if the radar type is valid
        validate.validate(radarType, version_name)
        if check_upload_file.check_upload_file(upload_file_path, version_name) is False:
            exit(1)
        temp_dir = tempfile.mkdtemp()
        # version_dir_to_upload= os.path.join(upload_file_path, version_name)
        zipped_file_path = os.path.join(temp_dir, version_name + '.zip')

        # print('version_dir_to_upload: ', version_dir_to_upload)
        print('zipped_file_path: ', zipped_file_path)
        
        create_zip.create_zip(zipped_file_path, version_name)
        push_software.push_software(zipped_file_path, radarType, version_name)
        remove_zip.remove_zip(zipped_file_path)
        exit(0)
    elif args.slack:
        slack_handler.send_slack_entrypoint()
        logger.log(message="Slack message sent successfully.", log_level="info", to_file=True, to_terminal=True)
        exit(0)
    else:
        parser.print_help()
        exit(1)








#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import subprocess
import os

def write_file_with_sudo(file_path, contents):
    with open(file_path, 'w') as file:
        file.write(contents)

def run_script_with_sudo(script_path, arguments):
    full_path = os.path.abspath(script_path)
    subprocess.run(['sudo', 'python3', full_path] + arguments, check=True)

def create_sudo_file(full_path, contents):
    # Ensure the full_path string is a string
    file_path = str(full_path)

    # Ensure the contents string is a string
    contents = str(contents)

    # Create a script that will write the file
    write_file_script = '/tmp/write_file_with_sudo.py'

    with open(write_file_script, 'w') as file:
        file.write(f'''
        
def write_file_with_sudo(file_path, contents):
    with open(file_path, 'w') as file:
        file.write(contents)
        
write_file_with_sudo({file_path!r}, {contents!r})
''')
    
    run_script_with_sudo(write_file_script, [])
    os.remove(write_file_script)

def rename_sudo_file(old_path, new_path):
    # Ensure the full_path string is a string
    old_path = str(old_path)
    new_path = str(new_path)

    # Create a script that will write the file
    rename_file_script = '/tmp/rename_file_with_sudo.py'

    with open(rename_file_script, 'w') as file:
        file.write(f'''

import os
        
def rename_file_with_sudo(old_path, new_path):
    os.rename(old_path, new_path)

rename_file_with_sudo({old_path!r}, {new_path!r})
''')

    run_script_with_sudo(rename_file_script, [])
    os.remove(rename_file_script)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import zipfile
from robin_sd_upload.supportive_scripts import logger

def create_zip(zip_name, folder_name):
    #check if folder exists
    if os.path.isdir(folder_name):
        logger.log(message="Folder exist: " + folder_name, log_level="info", to_file=True, to_terminal=True)
    else:
        return "Folder not exist: " + folder_name
    zipf = zipfile.ZipFile(zip_name, 'w', zipfile.ZIP_DEFLATED)
    for root, dirs, files in os.walk(folder_name):
        for file in files:
            zipf.write(os.path.join(root, file))
    zipf.close()


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import datetime

from robin_sd_upload.supportive_scripts import yaml_parser

def log(message: str, to_file: bool = True, to_terminal: bool = False, log_level: str = 'info'):
    """ Logs a message to a file and/or the terminal """
    # Usage: log("This is a test message", to_file=True, to_terminal=False)

    # Fetch the logfile location from  config
    config = yaml_parser.parse_config()

    # Check if the config was parsed correctly
    if config is None:
        raise Exception("Could not parse YAML config file")

    # check contents of config
    # print(config)

    log_file = config['log']['file']
    log_level = config['log']['level']
    app_name = config['static']['app_name']
    log_dir = os.path.dirname(log_file)

    # check if log_file or log_level or app_name is exist in config
    if log_file is None or log_level is None or app_name is None:
        raise Exception("log_file, log_level, or app_name is not defined in config")

    # Create the log directory if it doesn't exist
    if not os.path.exists(log_dir):
        os.makedirs(log_dir)
    
    # If the log file does not exist, create it.
    if not os.path.exists(log_file):
        open(log_file, 'w').close()

    # Fail if the log level is not valid, it can only be 'debug', 'info', 'warning', or 'error'
    if log_level not in ['debug', 'info', 'warning', 'error']:
        raise ValueError("log_level must be either 'debug', 'info', 'warning', or 'error'")

    if not isinstance(log_file, str) or not isinstance(message, str) or not isinstance(app_name, str):
        raise TypeError("log_file, message, and app_name must be of type string")
    if not isinstance(to_file, bool) or not isinstance(to_terminal, bool):
        raise TypeError("to_file and to_terminal must be of type boolean")

    # Create the log file if it doesn't exist
    if to_file and not os.path.exists(log_file):
        open(log_file, 'w').close()

    now = datetime.datetime.now().strftime("%H:%M:%S")
    log_line = f"{app_name} {now} - {message}"

    # Log to file
    if to_file:
        with open(log_file, 'a') as f:
            f.write(log_line + '\n')

    # Log to terminal
    if to_terminal:
        print(log_line)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
from robin_sd_upload.supportive_scripts import logger

#remove zip file if exists
def remove_zip(zip_name):
    if os.path.isfile(zip_name):
        os.remove(zip_name)
        logger.log(message="ZIP file removed: " + zip_name, log_level="info", to_file=True, to_terminal=True)
        # print("ZIP file removed: " + zip_name)
    else:
        return "ZIP not exist: " + zip_name


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
import shutil
from slack_sdk import WebClient
from slack_sdk.errors import SlackApiError

from robin_sd_upload.supportive_scripts import logger
from robin_sd_upload.supportive_scripts import yaml_parser

def zip_files(log_dir):
    # Zip the files
    try:
        zip_file = shutil.make_archive(log_dir, 'zip', log_dir)
        return zip_file
    except Exception as e:
        logger.log(message=f"Error zipping files: {e}", to_file=True, to_terminal=True, log_level='error')

def send_slack(file_to_send, customer_name: str = None):

    config = yaml_parser.parse_config()

    # Set variables from config
    slack_token = config['slack']['token']
    # slack_channel = config['slack']['channel']

    slack_channel = []
    slack_channel.append(config['slack']['channel'])
    
    client = WebClient(token=slack_token)

    try:
        response = client.files_upload(
            channels=slack_channel,
            file=file_to_send,
            filename=os.path.basename(file_to_send),
            title=f"SD API Uploader log files for {customer_name}",
            initial_comment=f"SD API Uploader log files for {customer_name}"
        )

        logger.log(message=f"Slack response: {response}", to_file=True, to_terminal=True, log_level='info')
        
    except SlackApiError as e:
        # You will get a SlackApiError if "ok" is False
        assert e.response["error"]    # str like 'invalid_auth', 'channel_not_found'
        logger.log(message=f"Slack error: {e.response['error']}", to_file=True, to_terminal=True, log_level='error')

def send_slack_entrypoint():
    config = yaml_parser.parse_config()

    customer_name = config['customer']['name']

    log_dir = config['log']['file']
    log_dir = os.path.dirname(log_dir)

    # Zip the files
    zip_file = zip_files(log_dir)

    # Send the files to Slack
    send_slack(zip_file, customer_name=customer_name)

    # Delete the zip file
    os.remove(zip_file)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
from robin_sd_upload.supportive_scripts import logger

def create_folder(folder):
    if not os.path.exists(folder):
        os.makedirs(folder)
        logger.log(message="Folder created: " + folder, log_level="info", to_file=True, to_terminal=True)
        logger.log(message="try to run script again", log_level="info", to_file=True, to_terminal=True)
        logger.log(message="Exiting script", log_level="info", to_file=True, to_terminal=True)
        exit()
    else:
        logger.log(message="Folder already exist: " + folder, log_level="info", to_file=True, to_terminal=True)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import requests
from robin_sd_upload.supportive_scripts import yaml_parser
from robin_sd_upload.supportive_scripts import logger

def get_bearer_token():

    config = yaml_parser.parse_config()
    
    email_add = config['robin_email']
    password = config['robin_password']
    request_url = config['api_url']

    headers = {
        'Content-Type': 'application/json',
    }

    data = '{"email": "' + email_add + '", "password": "' + password + '"}'
    response = requests.post(request_url + '/api/auth/login', headers=headers, data=data)

    if response.status_code == 200:
        bearer_token = response.json()['token']
        return bearer_token
    else:
        logger.log(message="Failed to get bearer token.", log_level="error", to_file=True, to_terminal=True)
        logger.log(message="Response code: " + str(response.status_code), log_level="error", to_file=True, to_terminal=True)
        logger.log(message="Response text: " + response.text, log_level="error", to_file=True, to_terminal=True)
        logger.log(message="Response headers: " + str(response.headers), log_level="error", to_file=True, to_terminal=False)
        logger.log(message="Data: " + data, log_level="error", to_file=True, to_terminal=False)
        logger.log(message="URL: " + request_url + '/api/auth/login', log_level="error", to_file=True, to_terminal=False)
        logger.log(message="Headers: " + str(headers), log_level="error", to_file=True, to_terminal=False)
        return 1
    


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages

with open("README.md") as file:
    long_description = file.read()

with open("requirements.txt") as file:
    install_requires = file.read().strip().split("\n")

# Fetch version from __version__ variable in robin_sd_upload/_version.py
from robin_sd_upload._version import __version__ as version

setup(
    name="robin_sd_upload",
    packages=find_packages(),
    install_requires=install_requires,
    entry_points={
        'console_scripts': [
            'robin-sd-upload = robin_sd_upload.__main__:main',
        ],
    },
    version=version,
    license="MIT",
    description="Package to upload files to the Robin Radar API",
    long_description=long_description,
    long_description_content_type="text/markdown",
    author="Robin Radar Systems",
    author_email="it-team@robinradar.com",
    url="https://bitbucket.org/robin-radar-systems/sd-api-upload-pip-package.git",
    keywords=["python", "robin", "radar", "upload", "software", "sd"]
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import validators
from robin_sd_upload.supportive_scripts import logger

def validate(radarType, version_name):
    radarTypearray = ["elvira", "iris", "birdradar"]
    if validators.length(radarType, min=1, max=50) == False:
        logger.log(message="Radar type is not valid: " + radarType, log_level="error", to_file=True, to_terminal=True)
        # print("Radar type is not valid: " + radarType)
        exit()
    if radarType not in radarTypearray:
            logger.log(message="Radar type is not valid: " + radarType, log_level="error", to_file=True, to_terminal=True)    
            # print("Radar type is not valid: " + radarType)
            exit()
    else:
        logger.log(message="Radar type is valid: " + radarType, log_level="info", to_file=True, to_terminal=True)
        # print("Radar type is valid: " + radarType)
        
    if validators.length(version_name, min=1, max=20) == False:
        logger.log(message="Version name is not valid: " + version_name, log_level="error", to_file=True, to_terminal=True)
        # print("Version name is not valid: " + version_name)
        exit()
    else:
        logger.log(message="Version name is valid: " + version_name, log_level="info", to_file=True, to_terminal=True)
        # print("Version name is valid: " + version_name)
        return radarType, version_name



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import os
from robin_sd_upload.supportive_scripts import logger

def check_upload_file(upload_file, version_name):
    # make sure not double / in path
    if upload_file.endswith("/"):
        upload_file = upload_file[:-1]
    # creating path from upload_file and version_name
    upload_file = upload_file + "/" + version_name
    logger.log(message="Checking upload file: " + upload_file, log_level="info", to_file=True, to_terminal=True)
    if os.path.isdir(upload_file):
        logger.log(message="File is valid and exist: " + upload_file, log_level="info", to_file=True, to_terminal=True)
        for root, dirs, files in os.walk(upload_file):
            for file in files:
                if file == "Packages.gz" or file == "Packages":
                    logger.log(message="Packages file found: " + file, log_level="info", to_file=True, to_terminal=True)
                    return True
                else:
                    logger.log(message="Packages file not found", log_level="error", to_file=True, to_terminal=True)
                    return False
    else:
        logger.log(message="File not exist: " + upload_file, log_level="error", to_file=True, to_terminal=True)
        return False
